from django.conf import settings

from pyforms.controls import ControlCheckBox
from pyforms_web.web.middleware import PyFormsMiddleware
from pyforms_web.widgets.django import ModelAdminWidget

from people.models import GroupMembership


class MembershipsListWidget(ModelAdminWidget):

    MODEL = GroupMembership

    LIST_ROWS_PER_PAGE = 50

    LIST_DISPLAY = [
        'person',
        'position',
        'date_joined',
        'date_left',
    ]

    FIELDSETS = [
        ('person', 'position', 'date_joined', 'date_left'),
    ]

    def __init__(self, *args, **kwargs):

        self._active_filter = ControlCheckBox(
            'Only active',
            default=True,
            label_visible=False,
            changed_event=self.populate_list,
        )
        super().__init__(*args, **kwargs)

    def get_toolbar_buttons(self, has_add_permission=False):
        return ('_add_btn' if has_add_permission else None, '_active_filter')

    def get_queryset(self, request, qs):
        if self._active_filter.value:
            qs = qs.active()
        return qs

    def has_add_permissions(self):
        user = PyFormsMiddleware.user()
        user_is_hr = settings.PROFILE_HUMAN_RESOURCES in user.groups.all()
        return user.is_superuser or user_is_hr

    def has_view_permissions(self, obj):
        user = PyFormsMiddleware.user()
        user_is_hr = settings.PROFILE_HUMAN_RESOURCES in user.groups.all()
        return user.is_superuser or user_is_hr

    def has_update_permissions(self, obj):
        user = PyFormsMiddleware.user()
        user_is_hr = settings.PROFILE_HUMAN_RESOURCES in user.groups.all()
        return user.is_superuser or user_is_hr

    def has_remove_permissions(self, obj):
        user = PyFormsMiddleware.user()
        user_is_hr = settings.PROFILE_HUMAN_RESOURCES in user.groups.all()
        return user.is_superuser or user_is_hr



from django.db import models

from .group_queryset import GroupQuerySet

class Group(models.Model):
    """
    Represents a Group in the system
    Example: Fly Facillities, Lima
    """
    type        = models.ForeignKey('GroupType', verbose_name ='Type', on_delete=models.CASCADE)
    name        = models.CharField('Name', max_length=200)
    subject     = models.CharField('Subject', max_length=200)
    web         = models.URLField(verbose_name ='Web site', blank=True, null=True)
    people_img  = models.ImageField('People', upload_to="uploads/group/people_img", blank=True, null=True)
    logo_img    = models.ImageField('Thumbnail', upload_to="uploads/group/group_img", blank=True, null=True)
    description = models.TextField('Description', default='', blank=True, null=True)

    members     = models.ManyToManyField('Person', verbose_name = 'Members',through='GroupMembership')
    responsible = models.ForeignKey('Person', related_name ='Head', verbose_name ='Head', blank=True, null=True, on_delete=models.CASCADE)

    objects = GroupQuerySet.as_manager()

    class Meta:
        ordering = ['name',]
        verbose_name = "Group"
        verbose_name_plural = "Groups"

    def __str__(self):
        return self.name

    @staticmethod
    def autocomplete_search_fields():
        return ("name__icontains",)




from confapp import conf
from people.models import Person
from permissions.models import Permission
from pyforms.controls import ControlCheckBox
from .humanresourses_form import HumanResourcesForm
from pyforms_web.widgets.django import ModelAdminWidget
from django.contrib.contenttypes.models import ContentType

class HumanResourcesList(ModelAdminWidget):
    """
    """
    UID   = 'hr-people'
    TITLE = 'People'

    MODEL = Person

    LIST_ROWS_PER_PAGE = 40

    LIST_HEADERS = ['Photo', 'Name', 'Email', 'Active']
    LIST_COLS_ALIGN = ['center', 'left', 'left', 'center']
    LIST_DISPLAY = (
        'thumbnail_80x80',
        'full_name',
        'email',
        'active',
    )

    SEARCH_FIELDS = (
        'full_name__icontains',
        'email__icontains',
        'personal_email__icontains'
    )

    EDITFORM_CLASS = HumanResourcesForm

    EXPORT_CSV = True
    EXPORT_CSV_COLUMNS = [
        # NOTE: These specific columns were asked by Teresa for a report
        'full_name',
        'email',
        'has_active_contract',
        'get_groups',
    ]

    # Orquestra Configuration
    # =========================================================================

    LAYOUT_POSITION = conf.ORQUESTRA_HOME

    USE_DETAILS_TO_EDIT = False

    ORQUESTRA_MENU = 'middle-left>HRDashboard'
    ORQUESTRA_MENU_ICON = 'users'
    ORQUESTRA_MENU_ORDER = 1


    @classmethod
    def has_permissions(cls, user):
        if user.is_superuser: return True

        # Search for the user groups with certain permissions
        contenttype = ContentType.objects.get_for_model(cls.MODEL)
        authgroups  = user.groups.filter(permissions__content_type=contenttype)
        authgroups  = authgroups.filter(permissions__codename='app_access_hr')
        perms = Permission.objects.filter(auth_group__in=authgroups)
        return perms.exists()

    def __init__(self, *args, **kwargs):

        self._active_filter = ControlCheckBox(
            'Only active',
            default=True,
            label_visible=False,
            changed_event=self.populate_list
        )

        self._nocontract_filter = ControlCheckBox(
            'No contract',
            default=False,
            label_visible=False,
            changed_event=self.populate_list
        )

        self._noproposal_filter = ControlCheckBox(
            'No proposal',
            default=False,
            label_visible=False,
            changed_event=self.populate_list
        )

        super().__init__(*args, **kwargs)

        if hasattr(self, '_add_btn'):
            self._add_btn.field_css = 'fifteen wide'

        self.__check_data_integraty()


    def get_queryset(self, request, qs):

        if self._active_filter.value:
            qs = qs.active()

        if self._nocontract_filter.value:
            qs = qs.noactivecontract()

        if self._noproposal_filter.value:
            qs = qs.noactiveproposal()

        return qs

    def get_toolbar_buttons(self, has_add_permission=False):
        return (
            '_add_btn' if has_add_permission else None,
            '_active_filter',
            '_nocontract_filter',
            '_noproposal_filter',
        )

    def __check_data_integraty(self):
        # FIXME needs update to show useful warnings
        return

        nostartdate = Person.objects.nostartdate()
        if nostartdate.exists():
            self.warning(
                ' <b>|</b> '.join( map(str, nostartdate) ),
                title='People without *joined date* set!')

        noenddate = Person.objects.filter(person_active=False).nostartdate()
        if noenddate.exists():
            self.warning(
                ' <b>|</b> '.join( map(str, noenddate) ),
                title='People without *end date* set!')



# Generated by Django 2.2.4 on 2019-08-06 15:16

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import people.models.person.person


class Migration(migrations.Migration):

    dependencies = [
        ('people', '0002_auto_20190805_1713'),
    ]

    operations = [
        migrations.AlterField(
            model_name='person',
            name='auth_user',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='person', to=settings.AUTH_USER_MODEL, verbose_name='User'),
        ),
        migrations.AlterField(
            model_name='person',
            name='curriculum_vitae',
            field=models.FileField(blank=True, help_text='This field will appear in the website.', null=True, upload_to=people.models.person.person.get_cv_upload_path, verbose_name='Short Vitae'),
        ),
        migrations.AlterField(
            model_name='person',
            name='img',
            field=models.ImageField(blank=True, help_text='This field will appear in the website.', max_length=150, upload_to=people.models.person.person.get_image_upload_path, verbose_name='Picture'),
        ),
    ]



from people.models import InstitutionAffiliation
from pyforms_web.widgets.django import ModelAdminWidget


class InstitutionAffiliationInline(ModelAdminWidget):

    MODEL = InstitutionAffiliation
    TITLE = 'Affiliation'

    LIST_DISPLAY = ['institution', 'date_joined', 'date_left']

    FIELDSETS = [
        ('institution', 'date_joined', 'date_left')
    ]




from permissions.models import Permission
from .humanresources_list import HumanResourcesList
from .people_form import PeopleFormWidget
from django.contrib.contenttypes.models import ContentType

class PeopleListWidget(HumanResourcesList):
    """
    People app for group responsibles.
    """
    UID   = 'people'
    TITLE = 'People'

    LIST_DISPLAY = (
        'thumbnail_80x80',
        'full_name',
        'email',
        'position',
    )

    ORQUESTRA_MENU = 'left'

    EDITFORM_CLASS = PeopleFormWidget

    @classmethod
    def has_permissions(cls, user):
        if user.is_superuser: return True

        # Search for the user groups with certain permissions
        contenttype = ContentType.objects.get_for_model(cls.MODEL)
        authgroups  = user.groups.filter(permissions__content_type=contenttype)
        authgroups  = authgroups.filter(permissions__codename='app_access_people')
        return Permission.objects.filter(auth_group__in=authgroups).exists()

    def has_add_permissions(self):
        return False

    def has_remove_permissions(self, obj):
        return False

    def get_queryset(self, request, qs):
        qs = qs.active()

        if self._nocontract_filter.value:
            qs = qs.noactivecontract()

        if self._noproposal_filter.value:
            qs = qs.noactiveproposal()

        return qs

    def get_toolbar_buttons(self, has_add_permission=False):
        return (
            '_add_btn' if has_add_permission else None,
            '_nocontract_filter',
            '_noproposal_filter',
        )



from django.db import models


class Position(models.Model):
    """
    Represents the position a person has in the institute
    Example: Principal Invertigator, Research Technician, PhD Student
    """
    name = models.CharField(max_length=200)

    @staticmethod
    def autocomplete_search_fields():
        return ("name__icontains", )

    class Meta:
        ordering = ['name', ]

    def __str__(self):
        return self.name



from django.db import models

class GroupType(models.Model):
    """
    Represents a Type of a Group in the system
    Example: Fellow, Staff
    """
    name = models.CharField('Name', max_length=200)   #: Name

    class Meta:
        ordering = ['name',]
        verbose_name = "Group type"
        verbose_name_plural = "Group types"

    def __str__(self):
        return self.name






from django.db import models

class AcademicCareer(models.Model):
    """
    Represents an Academic Career of a Person in the system
    """
    graduation_year = models.IntegerField(null=True)
    person = models.ForeignKey('people.Person', on_delete=models.CASCADE, )

    institution = models.ForeignKey(
        to='common.Institution',
        verbose_name='Institute',  # aka academic institution
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
    )

    degree = models.ForeignKey(
        to='Degree',
        null=True,
        blank=True,
        on_delete=models.PROTECT,
    )

    field_of_study = models.ForeignKey(
        to='ScientificArea',
        verbose_name='Field of Study',
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
    )

    class Meta:
        ordering = ['-graduation_year']
        verbose_name = "Academic Career"
        verbose_name_plural = "Academic Careers"

    def __str__(self):
        if not self.degree or not self.field_of_study:
            return f'Incomplete Academic record for {self.person}'
        else:
            return f'{self.person} is a {self.degree} in {self.field_of_study}'



from people.models import GroupMembership
from pyforms_web.widgets.django import ModelAdminWidget


class GroupsInline(ModelAdminWidget):
    MODEL = GroupMembership
    TITLE = 'Groups'

    LIST_DISPLAY = ['group', 'position', 'date_joined', 'date_left']

    FIELDSETS = [
        ('group', 'position', 'date_joined', 'date_left')
    ]



# Generated by Django 2.2.4 on 2019-08-06 17:13

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('people', '0003_auto_20190806_1516'),
    ]

    operations = [
        migrations.AlterField(
            model_name='person',
            name='auth_user',
            field=models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='person', to=settings.AUTH_USER_MODEL, verbose_name='User'),
        ),
    ]



from people.apps.people.humanresources_list import HumanResourcesList
from .descriptions.degrees_app import DegreesAdminApp
from .descriptions.scientific_areas_app import ScientificAreasAdminApp
from .groups.groups_list import GroupsListApp
from .descriptions.grouptype_app import GroupTypesAdminApp
from .descriptions.positions_app import PositionsAdminApp


from .people.people_list import PeopleListWidget
from .groups.mygroups_list import MyGroupsApp

from .search import SearchWidget

try:
    from .profile import UserProfileFormWidget
except:
    pass

from .dashboard import HRDashboard


from confapp import conf
from pyforms_web.widgets.django import ModelAdminWidget

from people.models import Position


class PositionsAdminApp(ModelAdminWidget):

    UID   = 'positions'
    TITLE = 'Positions'
    MODEL = Position

    SEARCH_FIELDS = ['name__icontains']
    FIELDSETS = ['name']
    LIST_DISPLAY = ['name']

    ########################################################
    #### ORQUESTRA CONFIGURATION ###########################
    ########################################################
    LAYOUT_POSITION = conf.ORQUESTRA_HOME
    ORQUESTRA_MENU = 'top>CommonDashboard'
    ORQUESTRA_MENU_ORDER = 0
    ORQUESTRA_MENU_ICON = 'address card'
    ########################################################

    AUTHORIZED_GROUPS = ['superuser']




from django.db import models

class InstitutionAffiliation(models.Model):
    """
    Represents a Person wich is a Member of that Group
    """
    person = models.ForeignKey(
        to='Person',
        related_name='membership',
        on_delete=models.CASCADE,
    )

    institution = models.ForeignKey(
        to='common.Institution',
        related_name='membership',
        on_delete=models.CASCADE,
    )

    date_joined = models.DateField('Joined', null=True, blank=True)
    date_left = models.DateField('Left', null=True, blank=True)



from confapp import conf
from pyforms_web.widgets.django import ModelAdminWidget

from people.models import GroupType

class GroupTypesAdminApp(ModelAdminWidget):

    UID   = 'group-types'
    TITLE = 'Group types'
    MODEL = GroupType

    SEARCH_FIELDS = ['name__icontains']
    FIELDSETS = ['name']
    LIST_DISPLAY = ['name']

    ########################################################
    #### ORQUESTRA CONFIGURATION ###########################
    ########################################################
    LAYOUT_POSITION      = conf.ORQUESTRA_HOME
    ORQUESTRA_MENU       = 'top>CommonDashboard'
    ORQUESTRA_MENU_ORDER = 0
    ORQUESTRA_MENU_ICON  = 'tags'
    ########################################################

    AUTHORIZED_GROUPS = ['superuser']
    
    


from .person.person import Person
from .person.institution_affiliation import InstitutionAffiliation
from .person.academic_career import AcademicCareer
from .person.degree import Degree
from .person.scientific_area import ScientificArea

from .group.group_type import GroupType
from .group.group import Group
from .group.position import Position
from .group.group_membership import GroupMembership



from confapp import conf
from django.db.models import Q
from people.models import Group
from people.models import Person
from pyforms.controls import ControlText
from pyforms.basewidget import BaseWidget
from pyforms.controls import ControlQueryList
from pyforms.controls import ControlAutoComplete


class SearchWidget(BaseWidget):

    MODEL = Person

    UID = 'search'
    TITLE = 'Find people'

    LIST_HEADERS = [
        'Name', 'Groups', 'Email', 'Extension', ''
    ]
    LIST_COLS_SIZES = [
        '40%', '35%', '15%', '5%', '5%'
    ]
    LIST_DISPLAY = [
        'name',
        'get_groups',
        'email',
        'phone_extension',
        'thumbnail_80x80',
    ]

    LIST_FILTER = [
        'groupmember__group',
        'groupmember__position',
    ]

    # Orquestra Configuration
    # =========================================================================

    LAYOUT_POSITION = conf.ORQUESTRA_HOME

    ORQUESTRA_MENU = 'left'
    ORQUESTRA_MENU_ICON = 'search'
    ORQUESTRA_MENU_ORDER = 0

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self._search = ControlText('Search by name', on_enter_event=self.populate_list )
        self._groups = ControlAutoComplete("Filter by groups", queryset=Group.objects.all(), multiple=True, changed_event=self.populate_list)
        self._list   = ControlQueryList('People', list_display=self.LIST_DISPLAY, headers=self.LIST_HEADERS, columns_size=self.LIST_COLS_SIZES)

        self.formset = [
            '_search',
            '_groups',
            '_list'
        ]

        self.populate_list()
        
    def populate_list(self):
        qs = Person.objects.active()
        has_filter = False
        
        if self._search.value:
            qs = qs.filter(
                Q(full_name__icontains=self._search.value) |
                Q(email__icontains=self._search.value)
            )
            has_filter = True
        
        if self._groups.value:
            qs = qs.filter_by_groups(self._groups.value)
            has_filter = True

        if has_filter:
            qs = qs.order_by('full_name')
        else:
            qs = qs.order_by('?')

        self._list.value = qs



from confapp import conf
from django.conf import settings
from people.models import Person
from pyforms.basewidget import segment
from pyforms.controls import ControlImg
from .memberships import MembershipsListWidget
from pyforms_web.widgets.django import ModelFormWidget

try:
    from research.apps.groups.projects import ProjectsListWidget
    research_module_installed = 'research' in settings.INSTALLED_APPS
except:
    research_module_installed = False

class GroupFormApp(ModelFormWidget):

    LAYOUT_POSITION = conf.ORQUESTRA_NEW_TAB
    HAS_CANCEL_BTN_ON_EDIT = False
    CLOSE_ON_REMOVE = True
    TITLE = 'Group'



    INLINES = [MembershipsListWidget] + ([ProjectsListWidget] if research_module_installed else [])

    LAYOUT_POSITION = conf.ORQUESTRA_NEW_TAB

    FIELDSETS = [
        ('type', 'responsible'),
        ' ',
        (
            segment(
                'h3: IDENTIFICATION',
                'name',
                'subject',
                field_css='fourteen wide',
            ),
            segment(
                '_img',
                field_style='max-width:330px;'
            )
        ),
        {
            '1:Group Information': [
                'description', 'web', 'logo_img', '_grpimg',
            ],
            '2:Members': ['MembershipsListWidget'],
            '3:Projects': ['ProjectsListWidget'] if research_module_installed else None,
        }
    ]

    def __init__(self, *args, **kwargs):

        self._img = ControlImg('Image', label_visible=False)
        self._grpimg = ControlImg('Image', label_visible=False)

        super().__init__(*args, **kwargs)

        self.responsible.changed_event = self.__update_image_evt

        self.logo_img.changed_event = self.__update_group_img_evt

        self.__update_image_evt()
        self.__update_group_img_evt()

    def __update_group_img_evt(self):
        self._grpimg.value = self.logo_img.value

    def __update_image_evt(self):
        try:
            person = Person.objects.get(pk=self.responsible.value.pk)
            url = person.thumbnail_url('300x300')
        except:
            url = Person.DEFAULT_PICTURE_URL

        self._img.value = url

    @property
    def title(self):
        obj = self.model_object
        if obj is None:
            return ModelFormWidget.title.fget(self)
        else:
            return "Group: {0}".format(obj.name)

    @title.setter
    def title(self, value):
        ModelFormWidget.title.fset(self, value)



from confapp import conf
from pyforms_web.widgets.django import ModelAdminWidget

from people.models import Group as ResearchGroup

from .groups_form import GroupFormApp


class GroupsListApp(ModelAdminWidget):
    """
    """
    UID = 'groups'
    TITLE = 'Groups'

    MODEL = ResearchGroup

    ORQUESTRA_MENU = 'middle-left>HRDashboard'
    ORQUESTRA_MENU_ICON = 'rocket'
    ORQUESTRA_MENU_ORDER = 10

    LAYOUT_POSITION = conf.ORQUESTRA_HOME

    USE_DETAILS_TO_EDIT = False

    EDITFORM_CLASS = GroupFormApp

    LIST_DISPLAY = [
        'name',
        'subject',
        'responsible',
        'type',
    ]

    LIST_FILTER = ['type']

    SEARCH_FIELDS = ['name__icontains']

    @classmethod
    def has_permissions(cls, user):
        if user.is_superuser:
            return True

        # user is allowed if Coordinator / Admin / Manager of any group
        for group in user.groups.all():
            parts = group.name.split(': ')
            if (
                len(parts) == 3 and
                parts[0] == 'PROFILE' and
                parts[1] in ('Group Coordinator',
                             'Group Admin',
                             'Group Manager')
            ):
                return True
        else:
            return False



# Generated by Django 2.2.4 on 2019-08-05 17:13

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('common', '0001_initial'),
        ('people', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Degree',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=200, verbose_name='Degree')),
            ],
            options={
                'verbose_name': 'Degree',
                'verbose_name_plural': 'Degrees',
                'ordering': ['name'],
            },
        ),
        migrations.CreateModel(
            name='ScientificArea',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=100, verbose_name='Name')),
            ],
            options={
                'verbose_name': 'Scientific area',
                'verbose_name_plural': 'Scientific areas',
                'ordering': ['name'],
            },
        ),
        migrations.RenameField(
            model_name='person',
            old_name='joined_date',
            new_name='date_joined',
        ),
        migrations.RenameField(
            model_name='person',
            old_name='leave_date',
            new_name='date_left',
        ),
        migrations.CreateModel(
            name='AcademicCareer',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('graduation_year', models.IntegerField(null=True)),
                ('degree', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='people.Degree')),
                ('field_of_study', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='people.ScientificArea', verbose_name='Field of Study')),
                ('institution', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='common.Institution', verbose_name='Institute')),
                ('person', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='people.Person')),
            ],
            options={
                'verbose_name': 'Academic Career',
                'verbose_name_plural': 'Academic Careers',
                'ordering': ['-graduation_year'],
            },
        ),
    ]



from django.db import models
from django.utils import timezone
from django.db.models import Q

from django.contrib.contenttypes.models import ContentType

from permissions.models import Permission

class PersonQuerySet(models.QuerySet):

    def filter_by_groups(self, groups):
        return self.filter(group__in=groups)

    def active(self):
        """
        Filter only active people
        """
        return self.filter(active=True)

    def noactivecontract(self):
        """
        Filter all the people without active contracts
        """
        today = timezone.now().date()

        qs = self.extra(where=["""
            NOT EXISTS (SELECT 1 FROM humanresources_contract b WHERE b.id = people_person.id and b.contract_start<='{0}' and b.contract_end>='{0}')
        """.format(today.strftime('%Y-%m-%d'))
        ])
        return qs

    def noactiveproposal(self):
        """
        Filter all the people without active proposals
        """
        today = timezone.now().date()

        qs = self.extra(where=["""
            NOT EXISTS (SELECT 1 FROM humanresources_contractproposal c WHERE c.id = people_person.id and
            c.contractproposal_start<='{0}' and
            DATE_ADD(DATE_ADD(c.start, INTERVAL c.days_duration DAY), INTERVAL c.months_duration MONTH)>='{0}')
        """.format(today.strftime('%Y-%m-%d'))
        ])
        return qs

    def nostartdate(self):
        """
        Filter all people without start date
        """
        return self.filter(person_datejoined=None)

    def noenddate(self):
        """
        Filter all people without end date
        """
        return self.filter(person_end=None)

    # PyForms Querysets
    # =========================================================================

    def __query_permissions(self, user, permissions_filter):
        # Search for the user groups with certain permissions
        contenttype = ContentType.objects.get_for_model(self.model)
        authgroups  = user.groups.filter(permissions__content_type=contenttype)
        authgroups  = authgroups.filter(permissions_filter).distinct()
        return Permission.objects.filter(auth_group__in=authgroups)

    def __filter_by_permissions(self, user, perms):
        if user.is_superuser: return self

        if perms.exists():
            # check if the user has permissions to all people
            if perms.filter(researchgroup=None).exists():
                return self
            else:
                if perms.exists():
                    # check which research groups the user has to its people
                    groups_withaccess = [p.researchgroup for p in perms]
                    rankings = [(p.researchgroup, p.ranking) for p in perms]


                    rankfilters = Q()
                    for researchgroup, ranking in rankings:
                        rankfilters.add(Q(researchgroup=researchgroup, ranking__gte=ranking), Q.OR)
                    rankperms = Permission.objects.filter(rankfilters)

                    qs = self.exclude(
                        ~Q(auth_user=user) &
                        Q(auth_user__groups__rankedpermissions__in=rankperms)
                    )

                    now = timezone.now()
                    qs = qs.filter(
                        Q(auth_user=user) |
                        Q(
                            groupmember__date_joined__lte=now,
                            groupmember__date_left__gte=now,
                            groupmember__group__in=groups_withaccess
                        ) |
                        Q(
                            groupmember__date_joined__lte=now,
                            groupmember__date_left__isnull=True,
                            groupmember__group__in=groups_withaccess
                        ) |
                        Q(
                            groupmember__date_joined__isnull=True,
                            groupmember__date_left__isnull=True,
                            groupmember__group__in=groups_withaccess
                        )
                    )
                    return qs.distinct()

        # By default returns only the Person associated to the user
        return self.filter(auth_user=user).distinct()

    def list_permissions(self, user):
        # If super user then return all the People
        if user.is_superuser: return self

        # Search for the user groups with certain permissions
        perms = self.__query_permissions(user, Q(permissions__codename='view_person') | Q(permissions__codename='change_person'))
        return self.__filter_by_permissions(user, perms)


    def has_add_permissions(self, user):
        return True
        return self.__query_permissions(
            user,
            Q(permissions__codename='add_person')
        ).exists()

    def has_view_permissions(self, user):
        perms = self.__query_permissions( user, Q(permissions__codename='view_person') | Q(permissions__codename='change_person') )
        return self.__filter_by_permissions(user, perms).exists()

    def has_update_permissions(self, user):
        perms = self.__query_permissions( user,  Q(permissions__codename='change_person') )
        return self.__filter_by_permissions(user, perms).exists()

    def has_remove_permissions(self, user):
        perms = self.__query_permissions( user, Q(permissions__codename='delete_person') )
        return self.__filter_by_permissions(user, perms).exclude(auth_user=user).exists()



from confapp import conf
from .groups_list import GroupsListApp

class MyGroupsApp(GroupsListApp):

    UID = 'my-groups'
    LAYOUT_POSITION = conf.ORQUESTRA_HOME

    ORQUESTRA_MENU = 'left'
    ORQUESTRA_MENU_ORDER = 10
    ORQUESTRA_MENU_ICON = 'rocket'

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)


from PIL import Image
from sorl.thumbnail import delete

from confapp import conf

from django.utils import timezone
from django.conf import settings
from django.contrib.auth.models import User as AuthUser
from django.contrib.auth.models import Group as AuthGroup

from pyforms_web.widgets.django import ModelAdminWidget
from pyforms_web.widgets.django import ModelFormWidget
from pyforms.basewidget import segment
from pyforms.basewidget import no_columns
from pyforms.controls import ControlImg
from pyforms.controls import ControlButton

from people.models import InstitutionAffiliation
from people.models import AcademicCareer
from people.models import Person
from people.models import GroupMembership

from django.conf import settings

try:
    from humanresources.apps.people.privateinfo import PrivateInfoFormWidget
    from humanresources.apps.proposals import ContractProposalsListWidget
    from humanresources.apps.contracts import ContractsListWidget
    hr_module_installed = 'humanresources' in settings.INSTALLED_APPS
except:
    hr_module_installed = False



from .inline_academic_career import AcademicCareerInline
from .inline_auth_groups import AuthGroupsInline
from .inline_groups import GroupsInline
from .inline_institution_affiliation import InstitutionAffiliationInline










class HumanResourcesForm(ModelFormWidget):
    """
    The advanced version of the form should only be available to
    administrators.
    """
    LAYOUT_POSITION = conf.ORQUESTRA_NEW_TAB

    HAS_CANCEL_BTN_ON_EDIT = False
    CLOSE_ON_REMOVE        = True

    FIELDSETS = [
        no_columns('_privateinfo_btn', '_proposals_btn', '_contracts_btn', 'active'),
        ' ',
        'h3:PERSONAL INFORMATION',
        (
            segment(
                'full_name',
                ('first_name', 'last_name'),
                ('gender', 'birthday'),
                ('position', 'curriculum_vitae'),
                'biography',
                field_css='fourteen wide',
            ),
            segment(
                '_img',
                '_rotimg_btn',
                'img',
                field_style='max-width:330px;'
            )
        ),
        'h3:EDUCATION',
        segment('AcademicCareerInline'),
        'h3:CONTACT INFORMATION',
        segment(
            ('email', 'personal_email'),
            ('phone_number', 'phone_extension'),
            'emergency_contact',
            css='secondary'
        ),
        'h3:INSTITUTIONAL INFORMATION',
        segment(
            'InstitutionAffiliationInline',
            ('card_number', 'room'),
            'auth_user',
            css='secondary'
        ),
        'h3:GROUPS',
        segment('GroupsInline', css='blue'),
        'h3:AUTH GROUPS',
        segment('AuthGroupsInline', css='red')
    ]

    READ_ONLY = ['auth_user']

    INLINES = [
        AcademicCareerInline,
        InstitutionAffiliationInline,
        GroupsInline,
        AuthGroupsInline,
    ]

    AUTHORIZED_GROUPS = ['superuser']


    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        if hasattr(self, 'active'):
            self.active.label_visible = False
            self.active.field_style = 'text-align: right;margin-top:5px;'
            self.active.field_css = 'two wide'

        self._rotimg_btn = ControlButton(
            '<i class="icon undo" ></i>Rotate',
            default=self.__rotimg_evt,
            label_visible=False,
            style='margin-top:5px;',
            field_style='text-align: right;',
            css='mini')

        self._img = ControlImg('Image', label_visible=False)

        self.img.changed_event = self.__update_image_evt

        ###############################################
        ## Human Resources related buttons
        ###############################################
        self._privateinfo_btn = ControlButton(
            '<i class="icon lock" ></i>Private info',
            default=self.__privateinfo_btn_evt,
            label_visible=False,
            css='basic red'
        )
        self._contracts_btn = ControlButton(
            '<i class="icon file alternate" ></i>Contracts',
            default=self.__contracts_btn_evt,
            label_visible=False,
            css='basic'
        )
        self._proposals_btn = ControlButton(
            '<i class="icon file alternate outline" ></i>Proposals',
            default=self.__proposals_btn_evt,
            label_visible=False,
            css='basic'
        )

        # Only show this buttons if the humanresources modules is installed.
        if not hr_module_installed:
            self._privateinfo_btn.hide()
            self._contracts_btn.hide()
            self._proposals_btn.hide()
        else:
            if self.model_object is None:
                self._contracts_btn.hide()
                self._proposals_btn.hide()
                self._privateinfo_btn.hide()

        if self.model_object is not None:
            self.__update_image_evt()

    @classmethod
    def has_permissions(cls, user):
        from .humanresources_list import HumanResourcesList
        return HumanResourcesList.has_permissions(user)

    def __update_image_evt(self):
        url = self.model_object.thumbnail_url(geometry_string='300x300')
        self._img.value = url + '?t=' + str(timezone.now().timestamp())

    def __rotimg_evt(self):
        delete(self.img.filepath, delete_file=False)
        im = Image.open(self.img.filepath)
        rot = im.rotate(90)
        rot.save(self.img.filepath)
        self.__update_image_evt()

    def __privateinfo_btn_evt(self):
        obj = self.model_object
        if obj:
            privateinfo = obj.get_privateinfo()
            app = PrivateInfoFormWidget(
                pk=privateinfo.pk,
                title=str(privateinfo),
                has_cancel_btn=False,
            )
            app.LAYOUT_POSITION = conf.ORQUESTRA_NEW_TAB

    def __proposals_btn_evt(self):
        obj = self.model_object
        if not obj:
            return

        app = ContractProposalsListWidget(
            parent_pk=obj.pk,
            parent_model=Person,
            has_cancel_btn=False,
        )
        app.LAYOUT_POSITION = conf.ORQUESTRA_NEW_TAB

    def __contracts_btn_evt(self):
        obj = self.model_object
        if not obj:
            return

        app = ContractsListWidget(
            parent_pk=obj.pk,
            parent_model=Person,
            has_cancel_btn=False,
        )
        app.LAYOUT_POSITION = conf.ORQUESTRA_NEW_TAB

    @property
    def title(self):
        obj = self.model_object
        if obj is None:
            return ModelFormWidget.title.fget(self)
        else:
            return "Person: {0}".format(obj.name)

    @title.setter
    def title(self, value):
        ModelFormWidget.title.fset(self, value)


import warnings, os
from allauth.account.models import EmailAddress
from django.conf import settings
from django.contrib.auth.models import Group, User
from django.core.exceptions import ObjectDoesNotExist
from django.db import models

from django.utils.html import format_html
from django.utils.text import slugify
from sorl.thumbnail import get_thumbnail
from django.utils    import timezone

from .person_queryset import PersonQuerySet

def get_image_upload_path(instance, filename):
    return os.path.join('uploads', 'person', 'img', str(instance.pk), filename)

def get_cv_upload_path(instance, filename):
    return os.path.join('uploads', 'person', 'cv', str(instance.pk), filename)

class Person(models.Model):
    """
    Represents a Person in the system
    Example: Adam Kampff, Ana Elias
    """
    DEFAULT_PICTURE_URL = '/static/square-image.png'

    active = models.BooleanField('Active', default=True)

    birthday    = models.DateField('Birthday', blank=True, null=True, )
    gender      = models.CharField('Gender', max_length=1, choices=(('F', 'Female'), ('M', 'Male')))
    first_name  = models.CharField('First name', max_length=40, help_text='This field will appear in the website.')
    middle_name = models.CharField('Middle name', max_length=100, blank=True)
    last_name   = models.CharField('Last name', max_length=40, help_text='This field will appear in the website.')
    full_name   = models.CharField('Name', max_length=150)

    email = models.EmailField('Research email', help_text='This field will appear in the website.', unique=True)
    web   = models.URLField('Website', blank=True, null=True,
                          help_text='This field will appear in the website.')
    personal_email  = models.EmailField('Personal email', blank=True, null=True, unique=True)
    phone_extension = models.IntegerField('Phone extension', blank=True, null=True)
    phone_number    = models.CharField('Contact', max_length=20, blank=True, null=True,
                                    help_text='This field will appear in the website if the next field is checked.')
    emergency_contact = models.TextField('Emergency contacts', max_length=100, blank=True, null=True, default='')

    biography        = models.TextField('Biography', blank=True, null=True, default='',
                                        help_text='This field will appear in the website.')
    curriculum_vitae = models.FileField('Short Vitae', upload_to=get_cv_upload_path, blank=True, null=True,
                                        help_text='This field will appear in the website.')

    position    = models.ForeignKey('Position', blank=True, null=True, on_delete=models.CASCADE)
    img         = models.ImageField(upload_to=get_image_upload_path, max_length=150, blank=True, verbose_name='Picture', help_text='This field will appear in the website.')  #: Picture link
    room        = models.CharField('Room', max_length=10, blank=True, null=True)     #: Room Number
    card_number = models.IntegerField('Card number', blank=True, null=True)
    date_joined = models.DateField('Joined date', blank=True, null=True, )
    date_left   = models.DateField('Leave date', blank=True, null=True, )

    auth_user   = models.OneToOneField(
        'auth.User',
        blank=True, null=True, verbose_name='User', related_name='person',
        on_delete=models.SET_NULL, unique=True
    )

    objects = PersonQuerySet.as_manager()

    class Meta:
        ordering = ['first_name','last_name' ]
        verbose_name = "Person"
        verbose_name_plural = "People"

        permissions = (
            ("app_access_hr",  "Access [Human Resources] app"),
            ("app_access_people", "Access [People] app"),
        )

    def __str__(self):
        return self.name


    def save(self, *args, **kw):
        """
        the function will create a django user for new person added to
        the data base or for each person that has been changed/saved
        without a django user

        the function will only feed the first name, last name and email
        as feeded in the person data base, and will create auser name
        first name.last name

        the administrator will have to provide a password, permmissions
        and related group in order to give acssess to this user
        """
        if self.auth_user is None:
            if User.objects.filter(email=self.email).exists():
                #if there is a user in django with this email
                self.auth_user = User.objects.get(email=self.email)
            else:
                self.auth_user = User.objects.create_user(
                    ".".join(list(map(slugify, [self.first_name, self.last_name]))),
                    self.email,
                    last_login=timezone.now()
                )
            self.auth_user.is_active = True
            self.auth_user.is_staff  = True
            self.auth_user.first_name = self.first_name
            self.auth_user.last_name  = self.last_name
            self.auth_user.save()

            # Set the email as verified automatically
            if EmailAddress.objects.filter(email=self.auth_user.email, user=self.auth_user).exists():
                e = EmailAddress(user=self.auth_user, email=self.auth_user.email, verified=True, primary=True)
                e.save()

        # TODO the above code block needs testing and updating
        # TODO create method .join_guest_group() to reuse

        if self.active:
            self.auth_user.is_active = True
        else:
            # If Person is set as inactive, revoke all access permissions
            self.auth_user.is_active = False
            self.auth_user.is_staff  = False
            self.auth_user.is_superuser = False
            self.auth_user.user_permissions.clear()
            self.auth_user.groups.clear()
        # The active status can be reverted but permissions will require
        # manual assignment
        self.auth_user.save()

        super().save(*args, **kw)


    @staticmethod
    def autocomplete_search_fields():
        return ("first_name__icontains", "middle_name__icontains", 'last_name__icontains')

    def fullname(self):
        """To be deprecated in favour of `full_name` field."""
        warnings.warn(
            'Stop using fullname() method, use the full_name field instead',
            DeprecationWarning,
        )
        return self.full_name

    @property
    def name(self):
        return " ".join(filter(None, (self.first_name, self.last_name)))

    def get_privateinfo(self):
        from humanresources.models import PrivateInfo
        obj, created = PrivateInfo.objects.get_or_create(person=self)
        return obj

    def person_contracts(self):
        """
        Return a link "Contracts" to the "Contracts" page where a user with permission
        can read and edit the contracts of the selected person.
        If this person still dont have a contract, the function
        returns a "Not created yet" label.

        @type  self:    Person
        @rtype:         link
        @return:        link to the "contracts" page of that person
        """
        try:
            return format_html("""<a class="btn btn-warning" href='/humanresources/contract/?person_id=%s' ><i class="icon-folder-close icon-black"></i> Show contracts</a>""" % (self.pk))
        except ObjectDoesNotExist:
            return "Not created yet"
    person_contracts.short_description = 'Contracts'
    person_contracts.allow_tags = True

    def has_active_contract(self):
        """Returns True if at least one of its contracts is active."""
        return self.contract_set.active().exists()

    def get_groups(self):
        """
        Returns a string with the name of all Research Groups this
        Person is a member of.
        """
        return '; '.join([
            membership.group.name
            for membership in self.groupmembership_set.active().all()
        ])

    def thumbnail_url(self, geometry_string):
        if self.img:
            url = get_thumbnail(
                self.img,
                geometry_string,
                crop='center',
                format='PNG',
            ).url
        else:
            url = self.DEFAULT_PICTURE_URL
        return url

    def thumbnail_80x80(self):
        url = self.thumbnail_url(geometry_string='80x80')
        return "<img class='ui top aligned tiny image rounded' src='{url}''>".format(url=url)
    thumbnail_80x80.short_description = ' '

    def photo(self):
        if self.img:
            return format_html("""<a class='imageLink' target='_blank'href='%s'><img src='%s' width='50px' height='50px'  ></a>""" % (self.img.url, self.img.url))
        else:
            return ''
    photo.short_description = 'Picture'
    photo.allow_tags = True

    # Object Permissions

    def has_view_permission(self, user):
        if any((
            user.is_superuser,
            user == self.auth_user,
            user.groups.filter(name=settings.PROFILE_HUMAN_RESOURCES).exists(),
        )):
            return True
        else:
            return False

    def has_change_permission(self, user):
        if any((
            user.is_superuser,
            user == self.auth_user,
            user.groups.filter(name=settings.PROFILE_HUMAN_RESOURCES).exists(),
        )):
            return True
        else:
            return False

    ######################################################################################
    #### FILTERS #########################################################################
    ######################################################################################





from django.contrib.staticfiles.templatetags.staticfiles import static

from django.conf import settings
from confapp import conf
from pyforms_web.web.middleware import PyFormsMiddleware
from pyforms_web.widgets.django import ModelAdminWidget
from pyforms_web.widgets.django import ModelFormWidget
from pyforms.basewidget import segment
from pyforms.controls import ControlImg
from pyforms.controls import ControlSimpleLabel
from pyforms.controls import ControlEmptyWidget

from people.models import Person
from people.models import GroupMembership

from .people.inline_institution_affiliation import InstitutionAffiliationInline
from .people.inline_academic_career import AcademicCareerInline
from .people.inline_groups import GroupsInline

try:
    from humanresources.models import PrivateInfo
    from humanresources.apps.people.profile_privateinfo import ProfilePrivateInfoFormWidget
    hr_module_installed = 'humanresources' in settings.INSTALLED_APPS
except:
    hr_module_installed = False

class ProfileGroupsInline(GroupsInline):

    def has_add_permissions(self):
        return False

    def has_view_permissions(self, obj):
        return False

    def has_update_permissions(self, obj):
        return False

    def has_remove_permissions(self, obj):
        return False


class UserProfileFormWidget(ModelFormWidget):

    UID = 'profile'
    TITLE = 'My Profile'

    MODEL = Person

    LAYOUT_POSITION = conf.ORQUESTRA_HOME

    ORQUESTRA_MENU = 'top'
    ORQUESTRA_MENU_ICON = 'user'
    ORQUESTRA_MENU_ORDER = 1

    HAS_CANCEL_BTN_ON_EDIT = False

    READ_ONLY = (
        'full_name',
        'gender',
        'img',
        'card_number',
        'phone_extension',
        'position',
    )

    FIELDSETS = [
        segment(
            ('_summary', '_img',),
        ),
        'h3:PERSONAL INFORMATION',
        {
            '1:General': [
                ('gender', 'birthday'),
                ('personal_email', 'phone_number'),
                'curriculum_vitae',
                'emergency_contact',
            ],
            '2:Public': [
                # 'info:The information provided in the fields below will be shared publicly.',
                ('first_name', 'last_name'),
                'biography',
            ],
            '3:Private': ['_privateinfo'] if hr_module_installed else None,
            '4:Institutional': [
                ('card_number', 'phone_extension'),
                'ProfileGroupsInline',
            ],
            '5:Affiliation':[
                'InstitutionAffiliationInline',
            ],
            '6:Education': [
                'AcademicCareerInline',
            ],
        },
    ]

    INLINES = [
        InstitutionAffiliationInline,
        ProfileGroupsInline,
        AcademicCareerInline,
    ]

    def __init__(self, *args, **kwargs):
        user = PyFormsMiddleware.user()
        try:
            person = Person.objects.get(auth_user=user)
        except Person.DoesNotExist:
            person = Person.objects.get(email=user.email)
            person.auth_user=user
            person.save()

        super().__init__(pk=person.pk, *args, **kwargs)

        membership = GroupMembership.objects.filter(person=person).first()
        membership_html = (
            f'<h3><a>{membership.group}</a> - {membership.position}</h3>'
            if membership is not None
            else ''
        )
        s = (
            f'<h1>{person.full_name}</h1>'
            '<address>'
            f'{membership_html}'
            '\n'
            f'<a href="mailto:{person.email}" target="_blank">'
            f'<i class="envelope icon"></i>{person.email}</a>'
            '</address>'
        )
        self._summary = ControlSimpleLabel(
            default=s,
            label_visible=False,
            field_css='fourteen wide',
        )

        if hr_module_installed:
            self._privateinfo = ControlEmptyWidget(
                parent=self,
                name='_privateinfo',
                default=ProfilePrivateInfoFormWidget(
                    pk=person.get_privateinfo().pk,
                    has_cancel_btn=False,
                ),
            )
        
        try:
            img_url = self.model_object.thumbnail_url(geometry_string='300x300')
        except AttributeError:
            img_url = static('square-image.png')
        self._img = ControlImg('Image', default=img_url, label_visible=False,
                               field_css='three wide')

    def save_form_event(self, obj):
        self._privateinfo.value.save_form_event(self._privateinfo.value.model_object)
        super().save_form_event(obj)

    def has_remove_permissions(self):
        return False



from pyforms.basewidget import segment, no_columns
from pyforms_web.widgets.django import ModelFormWidget

from .humanresourses_form import HumanResourcesForm
from .inline_academic_career import AcademicCareerInline
from .inline_institution_affiliation import InstitutionAffiliationInline


class PeopleFormWidget(HumanResourcesForm):

    FIELDSETS = [
        'h3:PERSONAL INFORMATION',
        (
            segment(
                'full_name',
                ('first_name', 'last_name'),
                ('gender', 'birthday'),
                ('position', 'curriculum_vitae'),
                'biography',
                field_css='fourteen wide',
            ),
            segment(
                '_img',
                '_rotimg_btn',
                'img',
                field_style='max-width:330px;'
            ),
        ),
        'h3:EDUCATION',
        segment('AcademicCareerInline'),
        'h3:CONTACT INFORMATION',
        segment(
            ('email', 'personal_email'),
            ('phone_number', 'phone_extension'),
            'emergency_contact',
            css='secondary'
        ),
        'h3:INSTITUTIONAL INFORMATION',
        segment(
            'InstitutionAffiliationInline',
            ('card_number', 'room'),
            css='secondary'
        ),
    ]

    READ_ONLY = ['full_name', 'auth_user', 'email', 'card_number']

    INLINES = [
        AcademicCareerInline,
        InstitutionAffiliationInline,
    ]


    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    @classmethod
    def has_permissions(cls, user):
        from .people_list import PeopleListWidget
        return PeopleListWidget.has_permissions(user)

    def has_remove_permissions(self):
        return False

    @property
    def title(self):
        obj = self.model_object
        if obj is None:
            return ModelFormWidget.title.fget(self)
        else:
            return "Member: {0}".format(obj.name)

    @title.setter
    def title(self, value):
        ModelFormWidget.title.fset(self, value)



from people.models import AcademicCareer
from pyforms_web.widgets.django import ModelAdminWidget


class AcademicCareerInline(ModelAdminWidget):

    MODEL = AcademicCareer

    LIST_DISPLAY = ['institution', 'degree', 'field_of_study', 'graduation_year']

    FIELDSETS = [
        ('institution', 'degree', 'field_of_study', 'graduation_year')
    ]




# Generated by Django 2.2.4 on 2019-08-23 13:42

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('people', '0004_auto_20190806_1713'),
    ]

    operations = [
        migrations.AlterField(
            model_name='person',
            name='email',
            field=models.EmailField(help_text='This field will appear in the website.', max_length=254, unique=True, verbose_name='Research email'),
        ),
        migrations.AlterField(
            model_name='person',
            name='personal_email',
            field=models.EmailField(blank=True, max_length=254, null=True, unique=True, verbose_name='Personal email'),
        ),
    ]



from django.db import models
from .group_membership_queryset import GroupMembershipQuerySet

class GroupMembership(models.Model):
    """
    Represents a Person wich is a Member of that Group
    """
    person = models.ForeignKey('Person', on_delete=models.CASCADE)
    group  = models.ForeignKey('Group', on_delete=models.CASCADE)

    date_joined = models.DateField('Joined', null=True, blank=True)
    date_left   = models.DateField('Left', null=True, blank=True)

    position = models.ForeignKey(
        to='Position',
        null=True,
        blank=True,
        on_delete=models.CASCADE,
    )

    objects = GroupMembershipQuerySet.as_manager()

    def __str__(self):
        position = self.position or self.person.position
        return f'{self.person.name} is a {position} in {self.group}'



# Generated by Django 2.2.4 on 2019-08-05 17:00

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('common', '0001_initial'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Group',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=200, verbose_name='Name')),
                ('subject', models.CharField(max_length=200, verbose_name='Subject')),
                ('web', models.URLField(blank=True, null=True, verbose_name='Web site')),
                ('people_img', models.ImageField(blank=True, null=True, upload_to='uploads/group/people_img', verbose_name='People')),
                ('logo_img', models.ImageField(blank=True, null=True, upload_to='uploads/group/group_img', verbose_name='Thumbnail')),
                ('description', models.TextField(blank=True, default='', null=True, verbose_name='Description')),
            ],
            options={
                'verbose_name': 'Group',
                'verbose_name_plural': 'Groups',
                'ordering': ['name'],
            },
        ),
        migrations.CreateModel(
            name='GroupType',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=200, verbose_name='Name')),
            ],
            options={
                'verbose_name': 'Group type',
                'verbose_name_plural': 'Group types',
                'ordering': ['name'],
            },
        ),
        migrations.CreateModel(
            name='Position',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=200)),
            ],
            options={
                'ordering': ['name'],
            },
        ),
        migrations.CreateModel(
            name='Person',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('active', models.BooleanField(default=True, verbose_name='Active')),
                ('birthday', models.DateField(blank=True, null=True, verbose_name='Birthday')),
                ('gender', models.CharField(choices=[('F', 'Female'), ('M', 'Male')], max_length=1, verbose_name='Gender')),
                ('first_name', models.CharField(help_text='This field will appear in the website.', max_length=40, verbose_name='First name')),
                ('middle_name', models.CharField(blank=True, max_length=100, verbose_name='Middle name')),
                ('last_name', models.CharField(help_text='This field will appear in the website.', max_length=40, verbose_name='Last name')),
                ('full_name', models.CharField(max_length=150, verbose_name='Name')),
                ('email', models.EmailField(help_text='This field will appear in the website.', max_length=254, verbose_name='Research email')),
                ('web', models.URLField(blank=True, help_text='This field will appear in the website.', null=True, verbose_name='Website')),
                ('personal_email', models.EmailField(blank=True, max_length=254, null=True, verbose_name='Personal email')),
                ('phone_extension', models.IntegerField(blank=True, null=True, verbose_name='Phone extension')),
                ('phone_number', models.CharField(blank=True, help_text='This field will appear in the website if the next field is checked.', max_length=20, null=True, verbose_name='Contact')),
                ('emergency_contact', models.TextField(blank=True, default='', max_length=100, null=True, verbose_name='Emergency contacts')),
                ('biography', models.TextField(blank=True, default='', help_text='This field will appear in the website.', null=True, verbose_name='Biography')),
                ('curriculum_vitae', models.FileField(blank=True, help_text='This field will appear in the website.', null=True, upload_to='uploads/person/person_cv', verbose_name='Short Vitae')),
                ('img', models.ImageField(blank=True, help_text='This field will appear in the website.', max_length=150, upload_to='uploads/person/person_img', verbose_name='Picture')),
                ('room', models.CharField(blank=True, max_length=10, null=True, verbose_name='Room')),
                ('card_number', models.IntegerField(blank=True, null=True, verbose_name='Card number')),
                ('joined_date', models.DateField(blank=True, null=True, verbose_name='Joined date')),
                ('leave_date', models.DateField(blank=True, null=True, verbose_name='Leave date')),
                ('auth_user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='person', to=settings.AUTH_USER_MODEL, verbose_name='User')),
                ('position', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='people.Position')),
            ],
            options={
                'verbose_name': 'Person',
                'verbose_name_plural': 'People',
                'ordering': ['first_name', 'last_name'],
                'permissions': (('app_access_hr', 'Access [Human Resources] app'), ('app_access_people', 'Access [People] app')),
            },
        ),
        migrations.CreateModel(
            name='InstitutionAffiliation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('date_joined', models.DateField(blank=True, null=True, verbose_name='Joined')),
                ('date_left', models.DateField(blank=True, null=True, verbose_name='Left')),
                ('institution', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='membership', to='common.Institution')),
                ('person', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='membership', to='people.Person')),
            ],
        ),
        migrations.CreateModel(
            name='GroupMembership',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('date_joined', models.DateField(blank=True, null=True, verbose_name='Joined')),
                ('date_left', models.DateField(blank=True, null=True, verbose_name='Left')),
                ('group', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='people.Group')),
                ('person', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='people.Person')),
                ('position', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='people.Position')),
            ],
        ),
        migrations.AddField(
            model_name='group',
            name='members',
            field=models.ManyToManyField(through='people.GroupMembership', to='people.Person', verbose_name='Members'),
        ),
        migrations.AddField(
            model_name='group',
            name='responsible',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='Head', to='people.Person', verbose_name='Head'),
        ),
        migrations.AddField(
            model_name='group',
            name='type',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='people.GroupType', verbose_name='Type'),
        ),
    ]



from confapp import conf
from pyforms.basewidget import BaseWidget

class HRDashboard(BaseWidget):
    """
    """
    UID = 'hr-dashboard'
    TITLE = 'Human resources'

    ORQUESTRA_MENU_ORDER = 3
    ORQUESTRA_MENU = 'middle-left'
    ORQUESTRA_MENU_ICON = 'users'
    LAYOUT_POSITION = conf.ORQUESTRA_HOME



from django.db import models


class ScientificArea(models.Model):
    """
    Represents the Scientific Area of a person
    Example: Neuroscience, Medicine
    """
    name = models.CharField('Name', max_length=100)  #: Name

    class Meta:
        ordering = ['name',]
        verbose_name = "Scientific area"
        verbose_name_plural = "Scientific areas"


    def __str__(self):
        return self.name

    @staticmethod
    def autocomplete_search_fields():
        return ("name__icontains", )



from django.db import models


class Degree(models.Model):
    """
    Academic degree: MSc, PhD, MD, ...
    """
    name = models.CharField('Degree', max_length=200)

    class Meta:
        ordering = ['name',]
        verbose_name = "Degree"
        verbose_name_plural = "Degrees"

    def __str__(self):
        return self.name



from django.db import models
from django.utils import timezone
from django.db.models import Q

class GroupMembershipQuerySet(models.QuerySet):

    def active(self):
        """
        Filter only active people
        """
        today = timezone.localtime(timezone.now()).date()
        query = Q()
        query.add(Q(date_joined__lte=today, date_left__gte=today), Q.OR)
        query.add(Q(date_joined__lte=today, date_left=None), Q.OR)
        query.add(Q(date_joined=None, date_left__gte=today), Q.OR)
        query.add(Q(date_joined=None, date_left=None), Q.OR)
        return self.filter(query)




from django.conf import settings
from django.db import models
from django.db.models import Q

class GroupQuerySet(models.QuerySet):

    # User dependent Querysets
    # =========================================================================

    def owned_by(self, user):
        """
        Filters the Queryset to objects owned by the User or where
        he is referenced in a Foreign Key.

        This is by default what everyone sees if they have no permissions.
        """
        return self.filter(
            Q(responsible__auth_user=user)
        ).distinct()

    def managed_by(self, user, default=None):
        """
        Filters the Queryset to objects the user is allowed to manage
        given his Authorization Group profiles.

        Notes:
            RankedPermissions not used.
            `required_codenames` not used
        """

        if default is None:
            default = self.none()

        if user.is_superuser:
            return self

        if settings.PROFILE_HUMAN_RESOURCES in user.groups.all():
            # HR profile may manage all groups
            return self

        # check which groups the user is Coordinator / Admin / Manager
        research_groups = []
        for group in user.groups.all():
            parts = group.name.split(': ')
            if (
                len(parts) == 3 and
                parts[0] == 'PROFILE' and
                parts[1] in ('Group Coordinator',
                             'Group Admin',
                             'Group Manager')
            ):
                research_groups.append(parts[2])

        qs = self.filter(name__in=research_groups) or default.distinct()

        return qs

    # PyForms Querysets
    # =========================================================================

    def list_permissions(self, user):
        return self.managed_by(
            user,
            default=self.owned_by(user)
        )

    def has_add_permissions(self, user):
        return user.is_superuser

    def has_view_permissions(self, user):
        return self.list_permissions(user)

    def has_update_permissions(self, user):
        return self.list_permissions(user)

    def has_remove_permissions(self, user):
        return user.is_superuser



from django.contrib.auth.models import User, Group
from people.models import Person
from pyforms_web.widgets.django import ModelFormWidget, ModelAdminWidget
from django.conf import settings

class UserAuthGroupsForm(ModelFormWidget):
    """Minimalistic User form only able to modify User groups."""
    MODEL = User
    FIELDSETS = ['groups']

    def __init__(self, *args, **kwargs):
        """
        Since we are coming from AuthGroup model, we need to
        redirect the PKs before showing the form.
        """
        person = Person.objects.get(pk=kwargs['parent_pk'])
        user   = person.auth_user
        kwargs['pk']    = user.pk
        kwargs['model'] = self.MODEL

        super().__init__(*args, **kwargs)


class AuthGroupsInline(ModelAdminWidget):
    MODEL = Group
    TITLE = 'Django Authorization Groups'

    EDITFORM_CLASS = UserAuthGroupsForm

    LIST_DISPLAY = ['name']

    AUTHORIZED_GROUPS = ['superuser', settings.APP_PROFILE_HR_PEOPLE]

    def get_queryset(self, request, queryset):
        person = Person.objects.get(pk=self.parent_pk)
        user = person.auth_user
        if user is None:
            return Group.objects.filter(pk=0)
        else:
            return user.groups.all()

    def has_add_permissions(self):
        # should always be False, use the Django Admin to manage these groups
        return False

    def has_remove_permissions(self, obj):
        # should always be False, use the Django Admin to manage these groups
        return False
